@@ -107,8 +107,8 @@ void db_x509::remFromCont(const QModelIndex &idx)
 	QList<pki_x509 *> childs;
 
 	while (pki->childCount()) {
-		child = (pki_x509*)pki->childItems.takeFirst();
-		child->delSigner((pki_x509*)pki);
+		child = dynamic_cast<pki_x509*>(pki->childItems.takeFirst());
+		child->delSigner(dynamic_cast<pki_x509*>(pki));
 		new_parent = child->findIssuer();
 		insertChild(new_parent, child);
 		if (new_parent)
@@ -199,7 +199,6 @@ void db_x509::inToCont(pki_base *pki)
 	 * and use its childs if we are newer */
 	items = sqlSELECTpki<pki_x509>(
 		"SELECT x509super.item FROM x509super "
-		"JOIN public_keys ON x509super.pkey = public_keys.item "
 		"JOIN certs ON certs.item = x509super.item "
 		"WHERE certs.ca=1 AND x509super.subj_hash=? "
 		"AND x509super.key_hash=?",
@@ -288,7 +287,6 @@ void db_x509::writeIndex(const QString &fname, bool hierarchy) const
 {
 	if (hierarchy) {
 		QString dir = fname + "/";
-		dir = nativeSeparator(dir);
 		QList<pki_x509*> issuers = sqlSELECTpki<pki_x509>(
 			"SELECT DISTINCT issuer FROM certs WHERE issuer != item");
 		foreach(pki_x509 *ca, issuers) {
@@ -353,11 +351,11 @@ a1int db_x509::getUniqueSerial(pki_x509 *signer)
 
 pki_base *db_x509::insert(pki_base *item)
 {
-	pki_x509 *cert = (pki_x509 *)item;
-	pki_x509 *oldcert = (pki_x509 *)getByReference(cert);
+	pki_x509 *cert = dynamic_cast<pki_x509 *>(item);
+	pki_x509 *oldcert = dynamic_cast<pki_x509 *>(getByReference(cert));
 	if (oldcert) {
 		XCA_INFO(tr("The certificate already exists in the database as:\n'%1'\nand so it was not imported").arg(oldcert->getIntName()));
-		delete(cert);
+		delete cert;
 		return NULL;
 	}
 	insertPKI(cert);
@@ -671,7 +669,7 @@ void db_x509::store(QModelIndexList list)
 	types = usual << exportType() << types;
 	ExportDialog *dlg = new ExportDialog(mainwin, tr("Certificate export"),
 		tr("X509 Certificates ( *.pem *.cer *.crt *.p12 *.p7b )"), crt,
-		MainWindow::certImg, types);
+		QPixmap(":certImg"), types);
 	if (!dlg->exec()) {
 		delete dlg;
 		return;
@@ -1094,7 +1092,7 @@ void db_x509::caProperties(QModelIndex idx)
 	ui.days->setSuffix(QString(" ") + tr("days"));
 	ui.days->setMaximum(1000000);
 	ui.days->setValue(cert->getCrlDays());
-	ui.image->setPixmap(*MainWindow::certImg);
+	ui.image->setPixmap(QPixmap(":certImg"));
 
 	QVariant tmplId = cert->getTemplateSqlId();
 	pki_temp *templ = mainwin->temps->lookupPki<pki_temp>(tmplId);
