@@ -30,6 +30,8 @@ class db_x509req: public db_x509super
 		void toRequest(QModelIndex index);
 		void load();
 		QList<pki_x509req*> getAllRequests();
+		void resetX509count();
+		void setSigned(QModelIndex index, bool signe);
 
 	public slots:
 		void newItem(pki_temp *temp, pki_x509req *orig = NULL);
