@@ -10,6 +10,7 @@
 #include "func.h"
 #include "exception.h"
 #include "lib/asn1time.h"
+#include "lib/settings.h"
 #include "widgets/validity.h"
 #include "widgets/XcaWarning.h"
 #include <openssl/objects.h>
@@ -29,6 +30,7 @@
 #endif
 #include <QDir>
 #include <QFile>
+#include <QFileInfo>
 #include <QStringList>
 #include <QLabel>
 #include <QLineEdit>
@@ -51,27 +53,66 @@ QPixmap *loadImg(const char *name )
 	return new QPixmap(QString(":") + name);
 }
 
-QStringList getLibExtensions()
+const QStringList getLibExtensions()
 {
-	QStringList l;
+	return QStringList {
 #if defined(Q_OS_WIN32)
-	l << QString("*.dll") << QString("*.DLL");
+		QString("*.dll"), QString("*.DLL"),
 #elif defined(Q_OS_MAC)
-	l << QString("*.dylib") << QString("*.so");
+		QString("*.dylib"), QString("*.so"),
 #else
-	l << QString("*.so");
+		QString("*.so"),
 #endif
-	return l;
+	};
 }
 
+#if defined(Q_OS_WIN32)
+static QString xcaExeDir()
+{
+	QString dir;
+	wchar_t inst_dir[2048];
+	ULONG dwLength = ARRAY_SIZE(inst_dir);
+
+	dwLength = GetModuleFileNameW(0, inst_dir, dwLength - 1);
+	dir = QString::fromWCharArray(inst_dir, dwLength);
+	int bslash = dir.lastIndexOf("\\");
+	if (bslash > 0)
+		dir = dir.mid(0, bslash);
+	return QFileInfo(dir).canonicalFilePath();
+}
+
+static QString registryInstallDir()
+{
+	QString dir;
+	wchar_t inst_dir[2048] = L"";
+	ULONG len = sizeof inst_dir;
+
+	if (RegGetValueW(HKEY_LOCAL_MACHINE, L"Software\\xca",
+			L"Install_Dir64", RRF_RT_REG_SZ, NULL,
+			inst_dir, &len) != ERROR_SUCCESS)
+		return dir;
+
+	/* "len" is in octets */
+	len /= sizeof inst_dir[0];
+	/* "len" includes the trailing \0\0 */
+	dir = QString::fromWCharArray(inst_dir, len -1);
+	return QFileInfo(dir).canonicalFilePath();
+}
+#endif
+
 int portable_app()
 {
 	static int portable = -1;
+	QString f1, f2;
 	if (portable == -1) {
 #if defined(Q_OS_WIN32)
-		HKEY hKey;
-		portable = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\xca", 0,
-			KEY_WOW64_32KEY|KEY_READ, &hKey) != ERROR_SUCCESS;
+		f1 = registryInstallDir();
+		f2 = xcaExeDir();
+		/* f1 == f2 Registry entry of install dir exists and matches
+		 * path of this xca.exe -> Installed. Not the portable app
+		 */
+		portable = f1 == f2 ? 0 : 1;
+		qDebug() << "Portable:" << f1 << " != " << f2;
 #else
 		const char *p = getenv("XCA_PORTABLE");
 		portable = p && *p;
@@ -85,42 +126,27 @@ int portable_app()
  * (e.g. c:\Program Files\xca )
  */
 
-QString getPrefix()
+const QString getPrefix()
 {
 #if defined(Q_OS_WIN32)
-	static char inst_dir[100] = "";
-	char *p;
-	ULONG dwLength = 100;
-	HKEY hKey;
+	static QString inst_dir;
+	QString reg_dir;
 
-	if (inst_dir[0] != '\0') {
+	if (!inst_dir.isEmpty()) {
 		/* if we already once discovered the directory just return it */
-		return QString(inst_dir);
-	}
-	// fallback: directory of xca.exe
-	GetModuleFileName(0, inst_dir, dwLength - 1);
-	p = strrchr(inst_dir, '\\');
-	if (p) {
-		*p = '\0';
-		return QString(inst_dir);
+		return inst_dir;
 	}
+	inst_dir = xcaExeDir();
+
 	if (portable_app())
 		return QString(inst_dir);
-	p = inst_dir;
-	*p = '\0';
-	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\xca", 0,
-		KEY_WOW64_32KEY|KEY_READ, &hKey) != ERROR_SUCCESS)
-	{
-		XCA_WARN("Registry Key: 'HKEY_LOCAL_MACHINE\\Software\\xca' not found");
-		return QString(inst_dir);
-	}
-	if (RegQueryValueEx(hKey, "Install_Dir", NULL, NULL,
-			(unsigned char *)inst_dir, &dwLength) != ERROR_SUCCESS)
-	{
+
+	reg_dir = registryInstallDir();
+	if (reg_dir.isEmpty())
 		XCA_WARN("Registry Key: 'HKEY_LOCAL_MACHINE->Software->xca->Install_Dir' not found");
-	}
-	RegCloseKey(hKey);
-	return QString(inst_dir);
+	else
+		inst_dir = reg_dir;
+	return inst_dir;
 
 #elif defined(Q_OS_MAC)
 	// since this is platform-specific anyway,
@@ -137,41 +163,88 @@ QString getPrefix()
 
 }
 
-QString getHomeDir()
-{
-	QString hd;
 #if defined(Q_OS_WIN32)
+static QString specialFolder(int csidl)
+{
 	LPITEMIDLIST pidl = NULL;
 	TCHAR buf[255] = "";
-	if (SUCCEEDED(SHGetSpecialFolderLocation(NULL, CSIDL_PERSONAL, &pidl))) {
-		SHGetPathFromIDList(pidl, buf);
-	}
-	hd = buf;
-#else
-	hd = QDir::homePath();
-#endif
-	return hd;
-}
 
-QString getLibDir()
+	if (SUCCEEDED(SHGetSpecialFolderLocation(NULL, csidl, &pidl)))
+		SHGetPathFromIDList(pidl, buf);
+
+	qDebug() << "Special Folder" << csidl << buf;
+	return QFileInfo(buf).canonicalFilePath();
+}
+#endif
+
+const QString getHomeDir()
 {
-	QString hd;
 #if defined(Q_OS_WIN32)
-	LPITEMIDLIST pidl = NULL;
-	TCHAR buf[255] = "";
-		if (SUCCEEDED(SHGetSpecialFolderLocation(NULL, CSIDL_SYSTEM, &pidl))) {
-		SHGetPathFromIDList(pidl, buf);
-	}
-	hd = buf;
+	return portable_app() ? getPrefix() : specialFolder(CSIDL_PERSONAL);
 #else
-	hd = QString("/usr/lib");
+	return QDir::homePath();
 #endif
-	return hd;
 }
 
-QString getDocDir()
+/* For portable APP remove leading file name if it is
+ * the app directory.
+ */
+QString relativePath(QString path)
 {
-#if defined(Q_OS_WIN32) || defined (Q_OS_MAC)
+	QFileInfo fi_path(path);
+	QFileInfo fi_home(getHomeDir());
+
+	QString prefix = fi_home.absoluteFilePath();
+	path = fi_path.absoluteFilePath();
+
+	if (portable_app()) {
+		if (path.startsWith(prefix))
+			path = path.mid(prefix.length()+1);
+	}
+	return path;
+}
+
+const QString getLibDir()
+{
+#if defined(Q_OS_WIN32)
+	return specialFolder(CSIDL_SYSTEM);
+#else
+	QString ulib = "/usr/lib/";
+	QString lib = "/lib/";
+	QString multi;
+	QString hd = ulib;
+
+	QFile f(ulib + "pkg-config.multiarch");
+	if (f.open(QIODevice::ReadOnly)) {
+		QTextStream in(&f);
+		multi = in.readLine();
+		if (!multi.isEmpty())
+			multi += "/";
+	}
+	QStringList dirs; dirs
+		<< ulib + multi + "pkcs11/"
+		<< lib + multi + "pkcs11/"
+		<< ulib + "pkcs11/"
+		<< lib + "pkcs11/"
+		<< ulib + multi
+		<< lib + multi
+		<< ulib
+		<< lib;
+	foreach(QString dir, dirs) {
+		if (QDir(dir).exists()) {
+			hd = dir;
+			break;
+		}
+	}
+	return QFileInfo(hd).canonicalFilePath();
+#endif
+}
+
+const QString getDocDir()
+{
+#if defined(Q_OS_WIN32)
+	return getPrefix() + "\\html";
+#elif defined (Q_OS_MAC)
 	return getPrefix();
 #else
 	return QString(DOCDIR);
@@ -182,18 +255,12 @@ QString getDocDir()
 // user-controlled settings on the current platform
 // i.e. PROFILE\Application Data\xca on windows, HOME/.xca on UNIX,
 // ~/Library/Preferences/xca on Mac OS X
-QString getUserSettingsDir()
+const QString getUserSettingsDir()
 {
 	QString rv;
 #if defined(Q_OS_WIN32)
-	LPITEMIDLIST pidl = NULL;
-	TCHAR buf[255] = "";
-	if (SUCCEEDED(SHGetSpecialFolderLocation(NULL, CSIDL_APPDATA, &pidl))) {
-	SHGetPathFromIDList(pidl, buf);
-	}
-	rv = buf;
-	rv += QDir::separator();
-	rv += "xca";
+	rv = portable_app() ? getPrefix() + "/settings" :
+				specialFolder(CSIDL_APPDATA) + "/xca";
 #elif defined(Q_OS_MAC)
   #if QT_VERSION < 0x050000
 	rv = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
@@ -206,13 +273,20 @@ QString getUserSettingsDir()
 		QCoreApplication::applicationName();
   #endif
 #else
-	rv = QDir::homePath();
-	rv += QDir::separator();
-	rv += ".xca";
+	rv = QDir::homePath() + "/.xca";
 #endif
 	return rv;
 }
 
+const QString getI18nDir()
+{
+#if defined(Q_OS_WIN32)
+	return getPrefix() + "\\i18n";
+#else
+	return getPrefix();
+#endif
+}
+
 // Qt's open and save dialogs result in some undesirable quirks.
 // This function makes sure that a filename has the user-selected extension.
 QString getFullFilename(const QString & filename, const QString & selectedFilter)
@@ -227,15 +301,6 @@ QString getFullFilename(const QString & filename, const QString & selectedFilter
 	return rv;
 }
 
-QByteArray filename2bytearray(const QString &fname)
-{
-#if defined(Q_OS_WIN32)
-	return fname.toLocal8Bit();
-#else
-	return fname.toUtf8();
-#endif
-}
-
 QString filename2QString(const char *fname)
 {
 #if defined(Q_OS_WIN32)
@@ -260,7 +325,7 @@ QString hostId()
 	HKEY hKey;
 
 	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, REG_CRYPTO, 0,
-			KEY_WOW64_64KEY|KEY_READ, &hKey) != ERROR_SUCCESS) {
+			KEY_READ, &hKey) != ERROR_SUCCESS) {
 		XCA_WARN("Registry Key: '" REG_CRYPTO "' not found");
 	} else {
 		if (RegQueryValueEx(hKey, REG_GUID, NULL, NULL,
@@ -485,13 +550,32 @@ bool _ign_openssl_error(const QString txt, const char *file, int line)
 	return !errtxt.isEmpty();
 }
 
-QString formatHash(const unsigned char *md, unsigned size, bool colon)
+QString formatHash(const QByteArray &data, QString sep, int width)
 {
-	QString s, t;
-	for (unsigned j = 0; j < size; j++)
-		s += t.sprintf("%02X%s", md[j],
-				(j+1 == size) || !colon ? "" : ":");
-	return s;
+	return QString(data.toHex()).toUpper()
+			.replace(QRegExp(QString("(.{%1})(?=.)").arg(width)),
+				 QString("\\1") + sep);
+}
+
+QByteArray Digest(const QByteArray &data, const EVP_MD *type)
+{
+	unsigned int n;
+	unsigned char m[EVP_MAX_MD_SIZE];
+
+	EVP_Digest(data.constData(), data.size(), m, &n, type, NULL);
+	openssl_error();
+	return QByteArray((char*)m, (int)n);
+}
+
+QString fingerprint(const QByteArray &data, const EVP_MD *type)
+{
+	return formatHash(Digest(data, type),
+			Settings["fp_separator"], Settings["fp_digits"]);
+}
+
+void update_workingdir(const QString &file)
+{
+	Settings["workingdir"] = QFileInfo(file).absolutePath();
 }
 
 void inc_progress_bar(int, int, void *p)
@@ -510,46 +594,48 @@ QMap<int, QString> dn_translations;
 
 void dn_translations_setup()
 {
-#define TRANS(nid, text) dn_translations[(nid)] = QObject::tr(text)
-	TRANS(NID_countryName, "Country code");
-	TRANS(NID_stateOrProvinceName, "State or Province");
-	TRANS(NID_localityName, "Locality");
-	TRANS(NID_organizationName, "Organisation");
-	TRANS(NID_organizationalUnitName, "Organisational unit");
-	TRANS(NID_commonName, "Common name");
-	TRANS(NID_pkcs9_emailAddress, "E-Mail address");
-	TRANS(NID_serialNumber, "Serial number");
-	TRANS(NID_givenName, "Given name");
-	TRANS(NID_surname, "Surname");
-	TRANS(NID_title, "Title");
-	TRANS(NID_initials, "Initials");
-	TRANS(NID_description, "Description");
-	TRANS(NID_role, "Role");
-	TRANS(NID_pseudonym, "Pseudonym");
-	TRANS(NID_generationQualifier, "Generation Qualifier");
-	TRANS(NID_x500UniqueIdentifier, "x500 Unique Identifier");
-	TRANS(NID_name, "Name");
-	TRANS(NID_dnQualifier, "DN Qualifier");
-	TRANS(NID_pkcs9_unstructuredName, "Unstructured name");
-	TRANS(NID_pkcs9_challengePassword, "Challenge password");
+QMap<int, QString> D;
+D[NID_countryName] = QObject::tr("Country code");
+D[NID_stateOrProvinceName] = QObject::tr("State or Province");
+D[NID_localityName] = QObject::tr("Locality");
+D[NID_organizationName] = QObject::tr("Organisation");
+D[NID_organizationalUnitName] = QObject::tr("Organisational unit");
+D[NID_commonName] = QObject::tr("Common name");
+D[NID_pkcs9_emailAddress] = QObject::tr("E-Mail address");
+D[NID_serialNumber] = QObject::tr("Serial number");
+D[NID_givenName] = QObject::tr("Given name");
+D[NID_surname] = QObject::tr("Surname");
+D[NID_title] = QObject::tr("Title");
+D[NID_initials] = QObject::tr("Initials");
+D[NID_description] = QObject::tr("Description");
+D[NID_role] = QObject::tr("Role");
+D[NID_pseudonym] = QObject::tr("Pseudonym");
+D[NID_generationQualifier] = QObject::tr("Generation Qualifier");
+D[NID_x500UniqueIdentifier] = QObject::tr("x500 Unique Identifier");
+D[NID_name] = QObject::tr("Name");
+D[NID_dnQualifier] = QObject::tr("DN Qualifier");
+D[NID_pkcs9_unstructuredName] = QObject::tr("Unstructured name");
+D[NID_pkcs9_challengePassword] = QObject::tr("Challenge password");
 
-	TRANS(NID_basic_constraints, "Basic Constraints");
-	TRANS(NID_subject_alt_name, "Subject alternative name");
-	TRANS(NID_issuer_alt_name, "issuer alternative name");
-	TRANS(NID_subject_key_identifier, "Subject key identifier");
-	TRANS(NID_authority_key_identifier, "Authority key identifier");
-	TRANS(NID_key_usage, "Key usage");
-	TRANS(NID_ext_key_usage, "Extended key usage");
-	TRANS(NID_crl_distribution_points, "CRL distribution points");
-	TRANS(NID_info_access, "Authority information access");
-	TRANS(NID_netscape_cert_type, "Certificate type");
-	TRANS(NID_netscape_base_url, "Base URL");
-	TRANS(NID_netscape_revocation_url, "Revocation URL");
-	TRANS(NID_netscape_ca_revocation_url, "CA Revocation URL");
-	TRANS(NID_netscape_renewal_url, "Certificate renewal URL");
-	TRANS(NID_netscape_ca_policy_url, "CA policy URL");
-	TRANS(NID_netscape_ssl_server_name, "SSL server name");
-	TRANS(NID_netscape_comment, "Comment");
+D[NID_basic_constraints] = QObject::tr("Basic Constraints");
+D[NID_subject_alt_name] = QObject::tr("Subject alternative name");
+D[NID_issuer_alt_name] = QObject::tr("issuer alternative name");
+D[NID_subject_key_identifier] = QObject::tr("Subject key identifier");
+D[NID_authority_key_identifier] = QObject::tr("Authority key identifier");
+D[NID_key_usage] = QObject::tr("Key usage");
+D[NID_ext_key_usage] = QObject::tr("Extended key usage");
+D[NID_crl_distribution_points] = QObject::tr("CRL distribution points");
+D[NID_info_access] = QObject::tr("Authority information access");
+D[NID_netscape_cert_type] = QObject::tr("Certificate type");
+D[NID_netscape_base_url] = QObject::tr("Base URL");
+D[NID_netscape_revocation_url] = QObject::tr("Revocation URL");
+D[NID_netscape_ca_revocation_url] = QObject::tr("CA Revocation URL");
+D[NID_netscape_renewal_url] = QObject::tr("Certificate renewal URL");
+D[NID_netscape_ca_policy_url] = QObject::tr("CA policy URL");
+D[NID_netscape_ssl_server_name] = QObject::tr("SSL server name");
+D[NID_netscape_comment] = QObject::tr("Comment");
+
+dn_translations = D;
 }
 
 QString appendXcaComment(QString current, QString msg)
