@@ -27,7 +27,7 @@
 
 #include "openssl_compat.h"
 
-pkcs11_lib_list pkcs11::libs;
+pkcs11_lib_list pkcs11::libraries;
 
 pkcs11::pkcs11()
 {
@@ -44,53 +44,7 @@ pkcs11::~pkcs11()
 	}
 }
 
-pkcs11_lib *pkcs11::load_lib(const QString &fname)
-{
-	if (fname.isEmpty())
-		return NULL;
-	return libs.add_lib(fname);
-}
-
-void pkcs11::reload_libs(const QString &libnames)
-{
-	QMap<QString, pkcs11_lib *> store;
-
-	if (libnames.isEmpty()) {
-		remove_libs();
-		return;
-	}
-
-	for (pkcs11_lib_list::iterator i = libs.begin(); i != libs.end(); ++i)
-		store[(*i)->filename()] = *i;
-	libs.clear();
-
-	foreach(QString name, libnames.split('\n')) {
-		bool enable;
-		QString n = pkcs11_lib::name2File(name, &enable);
-		pkcs11_lib *l = store.take(n);
-		if (l) {
-			if (enable == l->isEnabled()) {
-				libs.append(l);
-			} else {
-				delete l;
-				l = NULL;
-			}
-		}
-		// NOT else
-		if (!l)
-			l = load_lib(name);
-
-		qDebug() << "REORDER:" << n << name
-			 << "Enabled:" << l->isEnabled()
-			 << "Loaded:" << l->isLoaded()
-			 << "Should:" << enable;
-	}
-	qDebug() << "Delete remainig Libs start";
-	qDeleteAll(store.begin(), store.end());
-	qDebug() << "Delete remainig Libs done";
-}
-
-void pkcs11::startSession(slotid slot, bool rw)
+void pkcs11::startSession(const slotid &slot, bool rw)
 {
 	CK_RV rv;
 	unsigned long flags = CKF_SERIAL_SESSION | (rw ? CKF_RW_SESSION : 0);
@@ -124,7 +78,7 @@ void pkcs11::getRandom()
 		qDebug("C_GenerateRandom: %s", pk11errorString(rv));
 }
 
-QList<CK_MECHANISM_TYPE> pkcs11::mechanismList(slotid slot)
+QList<CK_MECHANISM_TYPE> pkcs11::mechanismList(const slotid &slot)
 {
 	CK_RV rv;
 	CK_MECHANISM_TYPE *m;
@@ -149,7 +103,8 @@ QList<CK_MECHANISM_TYPE> pkcs11::mechanismList(slotid slot)
 	return ml;
 }
 
-void pkcs11::mechanismInfo(slotid slot, CK_MECHANISM_TYPE m, CK_MECHANISM_INFO *info)
+void pkcs11::mechanismInfo(const slotid &slot, CK_MECHANISM_TYPE m,
+						CK_MECHANISM_INFO *info)
 {
 	CK_RV rv;
 	CALL_P11_C(slot.lib, C_GetMechanismInfo, slot.id, m, info);
@@ -238,7 +193,7 @@ static QDialog *newPinPadBox()
 	box->setWindowTitle(XCA_TITLE);
 	QHBoxLayout *h = new QHBoxLayout(box);
 	QLabel *l = new QLabel();
-	l->setPixmap(*MainWindow::scardImg);
+	l->setPixmap(QPixmap(":scardImg"));
 	l->setMaximumSize(QSize(95, 40));
 	l->setScaledContents(true);
 	h->addWidget(l);
@@ -316,7 +271,7 @@ bool pkcs11::selectToken(slotid *slot, QWidget *w)
 	Ui::SelectToken ui;
 	QDialog *select_slot = new QDialog(w);
 	ui.setupUi(select_slot);
-	ui.image->setPixmap(*MainWindow::scardImg);
+	ui.image->setPixmap(QPixmap(":scardImg"));
 	ui.tokenBox->addItems(slotnames);
 	ui.buttonBox->button(QDialogButtonBox::Ok)->setText(QObject::tr("Select"));
 	select_slot->setWindowTitle(XCA_TITLE);
@@ -346,7 +301,7 @@ static QString newSoPinTxt = QObject::tr(
 static QString newPinTxt = QObject::tr(
 		"Please enter the new PIN for the token: '%1'");
 
-void pkcs11::changePin(slotid slot, bool so)
+void pkcs11::changePin(const slotid &slot, bool so)
 {
 	Passwd newPin, pinp;
 	QString pin;
@@ -375,7 +330,7 @@ void pkcs11::changePin(slotid slot, bool so)
 	logout();
 }
 
-void pkcs11::initPin(slotid slot)
+void pkcs11::initPin(const slotid &slot)
 {
 	Passwd newPin, pinp;
 	int ret = 1;
@@ -406,7 +361,7 @@ void pkcs11::initPin(slotid slot)
 	logout();
 }
 
-void pkcs11::initToken(slotid slot, unsigned char *pin, int pinlen,
+void pkcs11::initToken(const slotid &slot, unsigned char *pin, int pinlen,
 		QString label)
 {
 	CK_RV rv;
@@ -420,7 +375,7 @@ void pkcs11::initToken(slotid slot, unsigned char *pin, int pinlen,
 		pk11error(slot, "C_InitToken", rv);
 }
 
-tkInfo pkcs11::tokenInfo(slotid slot)
+tkInfo pkcs11::tokenInfo(const slotid &slot)
 {
 	CK_TOKEN_INFO token_info;
 	CK_RV rv;
