@@ -23,15 +23,15 @@ class pki_pkcs7: public pki_base
 		pki_pkcs7(const QString name = "");
 		virtual ~pki_pkcs7();
 
-		void signFile(pki_x509 *crt, QString filename);
+		void signFile(pki_x509 *crt, const QString &filename);
 		void signCert(pki_x509 *crt, pki_x509 *contCert);
-		void encryptFile(pki_x509 *crt, QString filename);
+		void encryptFile(pki_x509 *crt, const QString &filename);
 		void writeP7(XFile &file, bool PEM) const;
 		void fromPEM_BIO(BIO *bio, const QString &name);
 		void fload(const QString &fname);
 		pki_x509 *getCert(int x);
 		void addCert(pki_x509 *crt);
-		int numCert(); // number of certs;
+		int numCert();
 
 };
 
