@@ -29,8 +29,6 @@
 
 #include "openssl_compat.h"
 
-QPixmap *pki_scard::icon[1] = { NULL };
-
 void pki_scard::init(void)
 {
 	ownPass = ptPin;
@@ -356,7 +354,7 @@ pk11_attlist pki_scard::objectAttributes(bool priv) const
 	return attrs;
 }
 
-void pki_scard::deleteFromToken(slotid slot)
+void pki_scard::deleteFromToken(const slotid &slot)
 {
 	pkcs11 p11;
 	p11.startSession(slot, true);
@@ -378,7 +376,7 @@ void pki_scard::deleteFromToken(slotid slot)
 	p11.deleteObjects(pub_objects);
 }
 
-int pki_scard::renameOnToken(slotid slot, QString name)
+int pki_scard::renameOnToken(const slotid &slot, const QString &name)
 {
 	pkcs11 p11;
 	p11.startSession(slot, true);
@@ -405,7 +403,7 @@ int pki_scard::renameOnToken(slotid slot, QString name)
 	return 1;
 }
 
-void pki_scard::store_token(slotid slot, EVP_PKEY *pkey)
+void pki_scard::store_token(const slotid &slot, EVP_PKEY *pkey)
 {
 	QByteArray ba;
 	RSA *rsa;
@@ -617,7 +615,7 @@ bool pki_scard::find_key_on_card(slotid *slot) const
  * returns the slot ID in slot true on success */
 bool pki_scard::prepare_card(slotid *slot) const
 {
-	if (!pkcs11::loaded())
+	if (!pkcs11::libraries.loaded())
 		return false;
 
 	QString msg = tr("Please insert card: %1 %2 [%3] with Serial: %4").
@@ -657,7 +655,7 @@ public:
 	}
 };
 
-void pki_scard::generateKey_card(int type, slotid slot, int size,
+void pki_scard::generateKey_card(int type, const slotid &slot, int size,
 		int curve_nid, QProgressBar *bar)
 {
 	pk11_attlist atts;
@@ -830,7 +828,8 @@ bool pki_scard::isToken()
 
 QVariant pki_scard::getIcon(const dbheader *hd) const
 {
-	return hd->id == HD_internal_name ? QVariant(*icon[0]) : QVariant();
+	return hd->id == HD_internal_name ?
+		QVariant(QPixmap(":scardIco")) : QVariant();
 }
 
 bool pki_scard::visible() const
