@@ -21,8 +21,6 @@
 
 #include "openssl_compat.h"
 
-QPixmap *pki_x509::icon[5];
-
 pki_x509::pki_x509(X509 *c)
 	:pki_x509super()
 {
@@ -88,7 +86,7 @@ QSqlError pki_x509::insertSqlData()
 		  "VALUES (?, ?, ?, ?, ?, ?, ?)");
 	q.bindValue(0, sqlItemId);
 	q.bindValue(1, hash());
-	q.bindValue(2, (uint)getIssuerName().hashNum());
+	q.bindValue(2, getIssuerName().hashNum());
 	q.bindValue(3, getSerial().toHex());
 	q.bindValue(4, signer ? signer->getSqlItemId() : QVariant());
 	q.bindValue(5, (int)isCA());
@@ -387,7 +385,7 @@ void pki_x509::deleteFromToken()
 	pki_scard *card = dynamic_cast<pki_scard *>(privkey);
 	slotidList p11_slots;
 
-	if (!card || !pkcs11::loaded())
+	if (!card || !pkcs11::libraries.loaded())
 		return;
 
 	if (privkey && privkey->isToken()) {
@@ -414,7 +412,7 @@ pk11_attlist pki_x509::objectAttributes()
 	return attrs;
 }
 
-void pki_x509::deleteFromToken(slotid slot)
+void pki_x509::deleteFromToken(const slotid &slot)
 {
 	pkcs11 p11;
 	p11.startSession(slot, true);
@@ -436,7 +434,7 @@ void pki_x509::deleteFromToken(slotid slot)
 	p11.deleteObjects(objs);
 }
 
-int pki_x509::renameOnToken(slotid slot, QString name)
+int pki_x509::renameOnToken(const slotid &slot, const QString &name)
 {
 
 	pkcs11 p11;
@@ -544,7 +542,7 @@ bool pki_x509::canSign() const
 	pki_key *privkey = getRefKey();
 	if (!privkey || privkey->isPubKey())
 		return false;
-	if (privkey->isToken() && !pkcs11::loaded())
+	if (privkey->isToken() && !pkcs11::libraries.loaded())
 		return false;
 	return isCA();
 }
@@ -709,7 +707,7 @@ bool pki_x509::verify_only(pki_x509 *signer)
 
 bool pki_x509::verify(pki_x509 *signer)
 {
-	if ((issuerSqlId != QVariant()) || (signer == NULL))
+	if (getSigner() || !signer)
 		return false;
 	if (signer == this &&
 	    issuerSqlId == sqlItemId &&
@@ -956,6 +954,8 @@ QStringList pki_x509::icsVEVENT() const
 QStringList pki_x509::icsVEVENT_ca() const
 {
 	QStringList ics;
+	pki_crl *crl = NULL;
+
 	ics << icsVEVENT();
 	foreach(pki_base *p, childItems) {
 		pki_x509 *pki = static_cast<pki_x509 *>(p);
@@ -963,14 +963,18 @@ QStringList pki_x509::icsVEVENT_ca() const
 			ics << pki->icsVEVENT();
 	}
 
-	ics << pki_base::icsVEVENT(crlExpire,
-		tr("CRL Renewal of CA '%1' due").arg(getIntName()),
-		tr("The latest CRL issued by the CA '%1' will expire on %2.\n"
-		  "It is stored in the XCA database '%3'")
-			.arg(getIntName())
-			.arg(crlExpire.toPretty())
-			.arg(currentDB)
-	);
+	QList<pki_crl*> list = db_base::sqlSELECTpki<pki_crl>(
+		"SELECT item FROM crls WHERE issuer = ?",
+		QList<QVariant>() << QVariant(sqlItemId));
+
+	/* Get latest CRL */
+	foreach(pki_crl *pki, list) {
+		if (!crl || crl->getNextUpdate() < pki->getNextUpdate())
+			crl = pki;
+	}
+	if (crl)
+		ics << crl->icsVEVENT();
+
 	return ics;
 }
 
@@ -978,15 +982,17 @@ QVariant pki_x509::getIcon(const dbheader *hd) const
 {
 	int pixnum = 0;
 	bool ca;
-
+	QStringList icon_names {
+		":validcertIco", ":validcertkeyIco",
+		":invalidcertIco", ":invalidcertkeyIco"
+	};
 	switch (hd->id) {
 	case HD_cert_ca:
 		if (!caAndPathLen(&ca, NULL, NULL))
 			return QVariant();
 		if (!ca)
 			return QVariant();
-		pixnum = 4;
-		break;
+		return QVariant(QPixmap(":doneIco"));
 	case HD_internal_name:
 		if (hasPrivKey())
 			pixnum += 1;
@@ -996,7 +1002,7 @@ QVariant pki_x509::getIcon(const dbheader *hd) const
 	default:
 		return pki_x509super::getIcon(hd);
 	}
-	return QVariant(*icon[pixnum]);
+	return QVariant(QPixmap(icon_names[pixnum]));
 }
 
 bool pki_x509::visible() const
