@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2011 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2020 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -32,15 +32,14 @@ class pki_x509req : public pki_x509super
 
 	public:
 		extList getV3ext() const;
-		static QPixmap *icon[3];
 		pki_x509req(QString name = "");
-		void fromPEM_BIO(BIO *bio, QString name);
-		void fload(const QString fname);
-		void writeDefault(const QString fname);
+		void fromPEM_BIO(BIO *bio, const QString &name);
+		void fload(const QString &fname);
+		void writeDefault(const QString &dirname) const;
 		~pki_x509req();
 		void fromData(const unsigned char *p, db_header_t *head);
 		x509name getSubject() const;
-		void writeReq(const QString fname, bool pem);
+		void writeReq(XFile &file, bool pem) const;
 		void markSigned(bool signe);
 		X509_REQ *getReq()
 		{
