@@ -23,9 +23,9 @@ class CertTreeView: public X509SuperTreeView
 	}
 	void fillContextMenu(QMenu *menu, QMenu *subExport,
 			const QModelIndex &index, QModelIndexList indexes);
-	void setModel(db_x509 *model=NULL)
+	void setModel(QAbstractItemModel *model=NULL)
 	{
-		certs = model;
+		certs = dynamic_cast <db_x509*> (model);
 		X509SuperTreeView::setModel(model);
 	}
 
