@@ -44,21 +44,12 @@
 #include "PwDialog.h"
 #include "OpenDb.h"
 
-QPixmap *MainWindow::keyImg = NULL, *MainWindow::csrImg = NULL,
-	*MainWindow::certImg = NULL, *MainWindow::tempImg = NULL,
-	*MainWindow::nsImg = NULL, *MainWindow::revImg = NULL,
-	*MainWindow::appIco = NULL, *MainWindow::scardImg = NULL,
-	*MainWindow::doneIco = NULL, *MainWindow::warnIco = NULL;
-
 db_key *MainWindow::keys = NULL;
 db_x509req *MainWindow::reqs = NULL;
 db_x509	*MainWindow::certs = NULL;
 db_temp	*MainWindow::temps = NULL;
 db_crl	*MainWindow::crls = NULL;
 
-NIDlist *MainWindow::eku_nid = NULL;
-NIDlist *MainWindow::dn_nid = NULL;
-
 OidResolver *MainWindow::resolver = NULL;
 
 void MainWindow::enableTokenMenu(bool enable)
@@ -70,9 +61,8 @@ void MainWindow::enableTokenMenu(bool enable)
 
 void MainWindow::load_engine()
 {
-	pkcs11::reload_libs(Settings["pkcs11path"]);
-	//	Error(err);
-	enableTokenMenu(pkcs11::loaded());
+	pkcs11::libraries.load(Settings["pkcs11path"]);
+	enableTokenMenu(pkcs11::libraries.loaded());
 }
 
 void MainWindow::initResolver()
@@ -137,11 +127,6 @@ MainWindow::MainWindow(QWidget *parent)
 	EVP_add_digest_alias(SN_sha256,SN_dsa_with_SHA256);
 	EVP_add_digest_alias(SN_sha384,SN_ecdsa_with_SHA384);
 	EVP_add_digest_alias(SN_sha512,SN_ecdsa_with_SHA512);
-	/* read in all our own OIDs */
-	initOIDs();
-
-	eku_nid = read_nidlist("eku.txt");
-	dn_nid = read_nidlist("dn.txt");
 
 	setAcceptDrops(true);
 
@@ -155,6 +140,12 @@ MainWindow::MainWindow(QWidget *parent)
 	crlView->setMainwin(this, searchEdit);
 	keys = NULL; reqs = NULL; certs = NULL; temps = NULL; crls = NULL;
 
+	keyView->setIconSize(QPixmap(":keyIco").size());
+	reqView->setIconSize(QPixmap(":reqIco").size());
+	certView->setIconSize(QPixmap(":validcertIco").size());
+	tempView->setIconSize(QPixmap(":templateIco").size());
+	crlView->setIconSize(QPixmap(":crlIco").size());
+
 	dhgen = NULL;
 	dhgenBar = new QProgressBar();
 	check_oom(dhgenBar);
@@ -226,64 +217,17 @@ void MainWindow::setItemEnabled(bool enable)
 	foreach(QAction *a, acList) {
 		a->setEnabled(enable);
 	}
-	enableTokenMenu(pkcs11::loaded());
-}
-
-/* creates a new nid list from the given filename */
-NIDlist *MainWindow::read_nidlist(QString name)
-{
-	NIDlist nl;
-	name = QDir::separator() + name;
-
-	/* first try $HOME/xca/ */
-	nl = readNIDlist(getUserSettingsDir() + name);
-#if !defined(Q_OS_WIN32)
-#if !defined(Q_OS_MAC)
-	if (nl.count() == 0){
-		/* next is /etx/xca/... */
-		nl = readNIDlist(QString(ETC) + name);
-	}
-#endif
-#endif
-	if (nl.count() == 0) {
-		/* look at /usr/(local/)share/xca/ */
-		nl = readNIDlist(getPrefix() + name);
-	}
-	return new NIDlist(nl);
+	enableTokenMenu(pkcs11::libraries.loaded());
 }
 
 void MainWindow::init_images()
 {
-	keyImg = loadImg("bigkey.png");
-	csrImg = loadImg("bigcsr.png");
-	certImg = loadImg("bigcert.png");
-	tempImg = loadImg("bigtemp.png");
-	nsImg = loadImg("netscape.png");
-	revImg = loadImg("bigcrl.png");
-	scardImg = loadImg("bigscard.png");
-	appIco = loadImg("key.xpm");
-	doneIco = loadImg("done.png");
-	warnIco = loadImg("warn.png");
-	bigKey->setPixmap(*keyImg);
-	bigCsr->setPixmap(*csrImg);
-	bigCert->setPixmap(*certImg);
-	bigTemp->setPixmap(*tempImg);
-	bigRev->setPixmap(*revImg);
-	setWindowIcon(*appIco);
-	pki_evp::icon[0] = loadImg("key.png");
-	pki_evp::icon[1] = loadImg("halfkey.png");
-	pki_scard::icon[0] = loadImg("scard.png");
-	pki_x509req::icon[0] = loadImg("req.png");
-	pki_x509req::icon[1] = loadImg("reqkey.png");
-	pki_x509req::icon[2] = doneIco;
-	pki_x509::icon[0] = loadImg("validcert.png");
-	pki_x509::icon[1] = loadImg("validcertkey.png");
-	pki_x509::icon[2] = loadImg("invalidcert.png");
-	pki_x509::icon[3] = loadImg("invalidcertkey.png");
-	pki_x509::icon[4] = doneIco;
-	pki_x509super::icon[0] = doneIco;
-	pki_temp::icon = loadImg("template.png");
-	pki_crl::icon = loadImg("crl.png");
+	bigKey->setPixmap(QPixmap(":keyImg"));
+	bigCsr->setPixmap(QPixmap(":csrImg"));
+	bigCert->setPixmap(QPixmap(":certImg"));
+	bigTemp->setPixmap(QPixmap(":tempImg"));
+	bigRev->setPixmap(QPixmap(":revImg"));
+	setWindowIcon(QPixmap(":appIco"));
 }
 
 void MainWindow::read_cmdline(int argc, char *argv[])
@@ -432,7 +376,7 @@ void MainWindow::pastePem()
 void MainWindow::initToken()
 {
 	bool ok;
-	if (!pkcs11::loaded())
+	if (!pkcs11::libraries.loaded())
 		return;
 	try {
 		pkcs11 p11;
@@ -473,7 +417,7 @@ void MainWindow::initToken()
 
 void MainWindow::changePin(bool so)
 {
-	if (!pkcs11::loaded())
+	if (!pkcs11::libraries.loaded())
 		return;
 	try {
 		pkcs11 p11;
@@ -494,7 +438,7 @@ void MainWindow::changeSoPin()
 
 void MainWindow::initPin()
 {
-	if (!pkcs11::loaded())
+	if (!pkcs11::libraries.loaded())
 		return;
 	try {
 		pkcs11 p11;
@@ -517,7 +461,7 @@ void MainWindow::manageToken()
 	pki_x509 *cert = NULL;
 	ImportMulti *dlgi = NULL;
 
-	if (!pkcs11::loaded())
+	if (!pkcs11::libraries.loaded())
 		return;
 
 	try {
@@ -589,10 +533,6 @@ MainWindow::~MainWindow()
 	ERR_free_strings();
 	EVP_cleanup();
 	OBJ_cleanup();
-	if (eku_nid)
-		delete eku_nid;
-	if (dn_nid)
-		delete dn_nid;
 	delete dbindex;
 #ifdef MDEBUG
 	fprintf(stderr, "Memdebug:\n");
