@@ -105,9 +105,7 @@ NewX509::NewX509(QWidget *parent)
 	:QDialog(parent)
 {
 	int i;
-	eku_nid = *MainWindow::eku_nid;
-	dn_nid = *MainWindow::dn_nid;
-	aia_nid << OBJ_sn2nid("OCSP") << OBJ_sn2nid("caIssuers");
+
 	attr_nid << NID_pkcs9_unstructuredName << NID_pkcs9_challengePassword;
 	QStringList keys;
 
@@ -116,7 +114,7 @@ NewX509::NewX509(QWidget *parent)
 	/* temporary storage for creating temporary X509V3_CTX */
 	ctx_cert = NULL;
 	pkiSource = generated;
-	foreach(int nid, dn_nid)
+	foreach(int nid, distname_nid)
 		keys << QString(OBJ_nid2ln(nid));
 
 	extDNlist->setKeys(keys);
@@ -139,7 +137,7 @@ NewX509::NewX509(QWidget *parent)
 		tabnames << tabWidget->tabText(i);
 	}
 
-	nsImg->setPixmap(*MainWindow::nsImg);
+	nsImg->setPixmap(QPixmap(":nsImg"));
 
 	// are there any useable private keys  ?
 	newKeyDone(NULL);
@@ -172,13 +170,9 @@ NewX509::NewX509(QWidget *parent)
 	tempList->insertPkiItems(MainWindow::temps->getAllAndPredefs());
 
 	// setup Extended keyusage
-	foreach(int nid, eku_nid)
+	foreach(int nid, extkeyuse_nid)
 		ekeyUsage->addItem(OBJ_nid2ln(nid));
 
-	// setup Authority Info Access
-	foreach(int nid, aia_nid)
-		aiaOid->addItem(OBJ_nid2ln(nid));
-
 	// init the X509 v3 context
 	X509V3_set_ctx(&ext_ctx, NULL , NULL, NULL, NULL, 0);
 	X509V3_set_ctx_nodb(&ext_ctx);
@@ -197,58 +191,65 @@ NewX509::NewX509(QWidget *parent)
 	pt = none;
 	notAfter->setEndDate(true);
 
-	QMap<int, DoubleClickLabel*> nidLabel;
-	nidLabel[NID_subject_alt_name] = sanLbl;
-	nidLabel[NID_issuer_alt_name] = ianLbl;
-	nidLabel[NID_crl_distribution_points] = crldpLbl;
-	nidLabel[NID_info_access] = aiaLbl;
-	nidLabel[NID_netscape_base_url] = nsBaseLbl;
-	nidLabel[NID_netscape_revocation_url] = nsRevLbl;
-	nidLabel[NID_netscape_ca_revocation_url] = nsCaRevLbl;
-	nidLabel[NID_netscape_renewal_url] = nsRenewLbl;
-	nidLabel[NID_netscape_ca_policy_url] = nsCaPolicyLbl;
-	nidLabel[NID_netscape_ssl_server_name] = nsSslServerLbl;
-	nidLabel[NID_netscape_comment] = nsCommentLbl;
+	QMap<int, QWidget*> nidWidget;
+	nidWidget[NID_subject_alt_name] = sanLbl;
+	nidWidget[NID_issuer_alt_name] = ianLbl;
+	nidWidget[NID_crl_distribution_points] = crldpLbl;
+	nidWidget[NID_info_access] = aiaLbl;
+	nidWidget[NID_netscape_base_url] = nsBaseLbl;
+	nidWidget[NID_netscape_revocation_url] = nsRevLbl;
+	nidWidget[NID_netscape_ca_revocation_url] = nsCaRevLbl;
+	nidWidget[NID_netscape_renewal_url] = nsRenewLbl;
+	nidWidget[NID_netscape_ca_policy_url] = nsCaPolicyLbl;
+	nidWidget[NID_netscape_ssl_server_name] = nsSslServerLbl;
+	nidWidget[NID_netscape_comment] = nsCommentLbl;
 
-	foreach(int nid, nidLabel.keys()) {
-		DoubleClickLabel *l = nidLabel[nid];
-		l->setText(Settings["translate_dn"] ?
-			dn_translations[nid] : OBJ_nid2ln(nid));
-		if (l->toolTip().isEmpty()) {
-			l->setToolTip(Settings["translate_dn"] ?
-				OBJ_nid2ln(nid) : dn_translations[nid]);
-		}
-		l->setClickText(OBJ_nid2sn(nid));
-		connect(l, SIGNAL(doubleClicked(QString)),
-                        MainWindow::getResolver(), SLOT(searchOid(QString)));
-	}
+	nidWidget[NID_basic_constraints] = bcBox;
+	nidWidget[NID_key_usage] = kuBox;
+	nidWidget[NID_ext_key_usage] = ekuBox;
+	nidWidget[NID_netscape_cert_type] = nsCertTypeBox;
 
-	QMap<int, QGroupBox*> nidGroupBox;
-	nidGroupBox[NID_basic_constraints] = bcBox;
-	nidGroupBox[NID_key_usage] = kuBox;
-	nidGroupBox[NID_ext_key_usage] = ekuBox;
-	nidGroupBox[NID_netscape_cert_type] = nsCertTypeBox;
+	nidWidget[NID_subject_key_identifier] = subKey;
+	nidWidget[NID_authority_key_identifier] = authKey;
 
-	foreach(int nid, nidGroupBox.keys()) {
-		QGroupBox *g = nidGroupBox[nid];
-		g->setTitle(Settings["translate_dn"] ?
-			dn_translations[nid] : OBJ_nid2ln(nid));
-		if (g->toolTip().isEmpty()) {
-			g->setToolTip(Settings["translate_dn"] ?
-				OBJ_nid2ln(nid) : dn_translations[nid]);
+	foreach(int nid, nidWidget.keys()) {
+		QString text = OBJ_nid2ln(nid);
+		QString tooltip = dn_translations[nid];
+		QWidget *w = nidWidget[nid];
+		QString tt = w->toolTip();
+
+		if (Settings["translate_dn"])
+                        text.swap(tooltip);
+
+		if (!tt.isEmpty())
+			tooltip = QString("%1 (%2)").arg(tt).arg(tooltip);
+
+		w->setToolTip(tooltip);
+
+		DoubleClickLabel *l = dynamic_cast<DoubleClickLabel*>(w);
+		QGroupBox *g = dynamic_cast<QGroupBox*>(w);
+		QCheckBox *c = dynamic_cast<QCheckBox*>(w);
+		if (l) {
+			l->setText(text);
+			l->setClickText(OBJ_nid2sn(nid));
+			connect(l, SIGNAL(doubleClicked(QString)),
+				MainWindow::getResolver(),
+				SLOT(searchOid(QString)));
+		} else if (g) {
+			g->setTitle(text);
+		} else if (c) {
+			c->setText(text);
 		}
 	}
 
 	if (Settings["translate_dn"]) {
-		QList<QGroupBox*> gb;
-		gb << distNameBox << keyIdentBox;
+		QList<QGroupBox*> gb { distNameBox, keyIdentBox };
 		foreach(QGroupBox *g, gb) {
 			QString tt = g->toolTip();
 			g->setToolTip(g->title());
 			g->setTitle(tt);
 		}
-		QList<QCheckBox*> cbList;
-		cbList << bcCritical << kuCritical << ekuCritical;
+		QList<QCheckBox*> cbList { bcCritical,kuCritical,ekuCritical };
 		foreach(QCheckBox* cb, cbList) {
 			cb->setText(tr("Critical"));
 		}
@@ -261,6 +262,7 @@ NewX509::NewX509(QWidget *parent)
 	MAP_LE(subAltName);
 	MAP_LE(issAltName);
 	MAP_LE(crlDist);
+	MAP_LE(authInfAcc);
 	MAP_LE(nsComment);
 	MAP_LE(nsBaseUrl);
 	MAP_LE(nsRevocationUrl);
@@ -277,6 +279,7 @@ NewX509::NewX509(QWidget *parent)
 	MAP_CB(ekuCritical);
 	MAP_CB(subKey);
 	MAP_CB(authKey);
+	MAP_CB(OCSPstaple);
 	MAP_CB(validMidn);
 	MAP_CB(noWellDefinedExpDate);
 }
@@ -290,7 +293,7 @@ void NewX509::setRequest()
 	timewidget->setEnabled(false);
 	capt->setText(tr("Create Certificate signing request"));
 	authKey->setEnabled(false);
-	setImage(MainWindow::csrImg);
+	image->setPixmap(QPixmap(":csrImg"));
 	pt = x509_req;
 }
 
@@ -352,7 +355,7 @@ void NewX509::setTemp(pki_temp *temp)
 	tabWidget->removeTab(0);
 	privKeyBox->setEnabled(false);
 	validityBox->setEnabled(false);
-	setImage(MainWindow::tempImg);
+	image->setPixmap(QPixmap(":tempImg"));
 	pt = tmpl;
 	fromTemplate(temp);
 	comment->setPlainText(temp->getComment());
@@ -362,15 +365,10 @@ void NewX509::setTemp(pki_temp *temp)
 void NewX509::setCert()
 {
 	capt->setText(tr("Create x509 Certificate"));
-	setImage(MainWindow::certImg);
+	image->setPixmap(QPixmap(":certImg"));
 	pt = x509;
 }
 
-void NewX509::setImage(QPixmap *img)
-{
-	image->setPixmap(*img);
-}
-
 /* Select a template and apply it */
 void NewX509::defineTemplate(pki_temp *temp)
 {
@@ -533,7 +531,6 @@ void NewX509::extensionsFromTemplate(pki_temp *temp)
 	QString2lb(ekeyUsage, temp->getSetting("eKeyUse"));
 	validRange->setCurrentIndex(temp->getSettingInt("validM"));
 	nconf_data->document()->setPlainText(temp->getSetting("adv_ext"));
-	setAuthInfAcc_string(temp->getSetting("authInfAcc"));
 
 	on_applyTime_clicked();
 }
@@ -560,7 +557,6 @@ void NewX509::toTemplate(pki_temp *temp)
 		temp->setSetting(i.key(), i.value()->isChecked());
 	}
 
-	temp->setSetting("authInfAcc", getAuthInfAcc_string());
 	temp->setSetting("nsCertType", lb2int(nsCertType));
 	temp->setSetting("ca", basicCA->currentIndex());
 	temp->setSetting("keyUse", lb2int(keyUsage));
@@ -887,14 +883,7 @@ void NewX509::editV3ext(QLineEdit *le, QString types, int n)
 
 	dlg = new v3ext(this);
 	setupTmpCtx();
-	if (n == NID_info_access) {
-		int nid, idx = aiaOid->currentIndex();
-		if (idx >= 0 && idx < aia_nid.size()) {
-			nid = aia_nid[idx];
-			dlg->setPrefix(QString(OBJ_nid2sn(nid)) + ";");
-		}
-	}
-	dlg->addInfo(le, types.split(',' ), n, &ext_ctx);
+	dlg->addInfo(le, types.split(','), n, &ext_ctx);
 	dlg->exec();
 	delete(dlg);
 }
@@ -932,10 +921,10 @@ void NewX509::checkIcon(const QString &text, int nid, QLabel *img)
 		break;
 	}
 	if (ign_openssl_error()) {
-		img->setPixmap(*MainWindow::warnIco);
+		img->setPixmap(QPixmap(":warnIco"));
 		return;
 	}
-	img->setPixmap(*MainWindow::doneIco);
+	img->setPixmap(QPixmap(":doneIco"));
 }
 
 void NewX509::checkSubAltName(const QString & text)
@@ -1079,7 +1068,14 @@ void NewX509::on_editCrlDist_clicked()
 
 void NewX509::on_editAuthInfAcc_clicked()
 {
-	editV3ext(authInfAcc, "URI,email,RID,DNS,IP", NID_info_access);
+	QStringList permut, groups { "OCSP", "caIssuers" },
+			types{ "URI", "email", "RID", "DNS", "IP" };
+	foreach(QString group, groups) {
+		foreach(QString type, types) {
+			permut << QString("%1;%2").arg(group).arg(type);
+		}
+	}
+	editV3ext(authInfAcc, permut.join(","), NID_info_access);
 }
 
 void NewX509::on_tabWidget_currentChanged(int tab)
