@@ -49,7 +49,7 @@ class SearchPkcs11: public QDialog, public Ui::SearchPkcs11
 	searchThread *searching;
 
    public:
-	SearchPkcs11(QWidget *parent, QString fname);
+	SearchPkcs11(QWidget *parent, const QString &fname);
 	~SearchPkcs11();
 
    public slots:
