@@ -14,6 +14,11 @@
 #include "lib/db.h"
 #include "MainWindow.h"
 
+// index = enum pki_type
+static const char * const PixmapMap[] = {
+  "" ":keyImg", ":csrImg", ":certImg", ":revImg", ":tempImg", "", ":scardImg",
+};
+
 class XcaDialog : public QDialog, public Ui::XcaDialog
 {
 	QWidget *widg;
@@ -23,19 +28,7 @@ class XcaDialog : public QDialog, public Ui::XcaDialog
 	{
 		setupUi(this);
 		setWindowTitle(XCA_TITLE);
-
-		QPixmap *icon = NULL;
-		switch (type) {
-		case asym_key:   icon = MainWindow::keyImg; break;
-		case x509_req:   icon = MainWindow::csrImg; break;
-		case x509:       icon = MainWindow::certImg; break;
-		case revocation: icon = MainWindow::revImg; break;
-		case tmpl:       icon = MainWindow::tempImg; break;
-		case smartCard:  icon = MainWindow::scardImg; break;
-		default: break;
-		}
-		if (icon)
-			image->setPixmap(*icon);
+		image->setPixmap(QPixmap(PixmapMap[type]));
 		content->addWidget(w);
 		widg = w;
 		title->setText(t);
@@ -56,9 +49,8 @@ class XcaDialog : public QDialog, public Ui::XcaDialog
 			widg->setSizePolicy(QSizePolicy::Expanding,
 						QSizePolicy::Expanding);
 	}
-	void aboutDialog(QPixmap *lefticon)
+	void aboutDialog(const QPixmap &left)
 	{
-		QPixmap left = *lefticon;
 		title->setPixmap(left.scaledToHeight(title->height()));
 		noSpacer();
 		resize(560, 400);
